@@ -1,7 +1,7 @@
 #!/usr/bin/python -u
 
 # Console Jabber Client
-# Copyright (C) 2004-2005  Jacek Konieczny
+# Copyright (C) 2004-2009  Jacek Konieczny
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -35,6 +35,7 @@ import codecs
 
 import pyxmpp.all
 import pyxmpp.jabber.all
+import pyxmpp.exceptions
 from pyxmpp import jabber
 
 from cjc import cjclogging
@@ -48,6 +49,7 @@ from cjc import themes
 from cjc import common
 from cjc import tls
 from cjc import completions
+from cjc import cjc_globals
 
 class Exit(Exception):
     pass
@@ -92,7 +94,7 @@ global_theme_formats=(
     ("window_status",u"%[bar] %(active)s%(locked)s %(buffer_num)s: %{buffer_descr}"),
     ("default_buffer_descr",u"%(buffer_name)s"),
     ("title_bar",u"%[bar]%(program_name)s ver. %(program_version)s by %(program_author)s"),
-    ("status_bar",u"%[bar][%(J:me:show)s] (%(J:me:status)s) Active buffers: [%{buffers}]"),
+    ("status_bar",u"%[bar]<%{buffers}> [%(J:me:show)s] (%(J:me:status)s)"),
     ("error",u"%[error][%(T:now)s] %(msg)s\n"),
     ("warning",u"%[warning][%(T:now)s] %(msg)s\n"),
     ("info",u"%[info][%(T:now)s] %(msg)s\n"),
@@ -113,7 +115,7 @@ global_theme_formats=(
             "  Serial number: %(serial_number)s\n"
             "  Valid not before: %(not_before)s\n"
             "  Valid not after: %(not_after)s\n"),
-    ("certificate_error",u"%[warning]Server certificate failed verifiaction.\n\n"
+    ("certificate_error",u"%[warning]Server certificate failed verification.\n\n"
         "%[info]Server has presented following certificate chain:\n"
         "%{chain}\n"
         "The rejected certificate is:\n"
@@ -121,7 +123,7 @@ global_theme_formats=(
         "%[error]Verification failed with error #%(errnum)i: %(errdesc)s\n"),
     ("certificate_remember",u"%[info]Server '%(who)s' has presented following certificate:\n\n"
         "%{certificate}\n"
-        "Verification of that certificat failed.\n"
+        "Verification of that certificate failed.\n"
         "Should it be accepted in future sessions anyway?\n"),
     ("tls_error_ignored",u"%[warning]Certificate verification error #%(errnum)i:"
             " '%(errdesc)s' ignored - peer certificate is known as trustworthy.\n"),
@@ -131,8 +133,11 @@ global_theme_formats=(
 
 
 class Application(tls.TLSMixIn,jabber.Client):
-    def __init__(self, base_dir, config_file="default", theme_file="theme", 
-            home_dir=None,profile=False):
+    def __init__(self, base_dir, config_file = "default", theme_file = "theme", 
+            home_dir = None, profile = False):
+        if cjc_globals.application is not None:
+            raise "An Application instance already present"
+        cjc_globals.application = self
         self.profile=profile
         tls.TLSMixIn.__init__(self)
         jabber.Client.__init__(self, disco_name="CJC", disco_type="console")
@@ -365,83 +370,83 @@ class Application(tls.TLSMixIn,jabber.Client):
 
     def layout_plain(self):
         ui_buffer.activity_handlers=[]
-        self.top_bar=ui.StatusBar(self.theme_manager,"title_bar",{})
+        self.top_bar=ui.StatusBar("title_bar",{})
         ui_buffer.activity_handlers.append(self.top_bar.update)
-        self.main_window=ui.Window(self.theme_manager,"Main")
-        self.command_line=ui.Input(self.theme_manager)
-        self.bottom_bar=ui.StatusBar(self.theme_manager,"status_bar",{})
+        self.main_window=ui.Window("Main")
+        self.command_line=ui.Input()
+        self.bottom_bar=ui.StatusBar("status_bar",{})
         ui_buffer.activity_handlers.append(self.bottom_bar.update)
         sp=ui.HorizontalSplit(self.top_bar,self.main_window,self.bottom_bar,self.command_line)
-        self.screen.set_content(sp)
-        self.screen.focus_window(self.main_window)
+        cjc_globals.screen.set_content(sp)
+        cjc_globals.screen.focus_window(self.main_window)
         self.status_window=None
         self.roster_window=None
 
     def layout_icr(self):
         ui_buffer.activity_handlers=[]
-        self.top_bar=ui.StatusBar(self.theme_manager,"title_bar",{})
+        self.top_bar=ui.StatusBar("title_bar",{})
         ui_buffer.activity_handlers.append(self.top_bar.update)
-        self.status_window=ui.Window(self.theme_manager,"Status",1)
-        self.main_window=ui.Window(self.theme_manager,"Main")
-        self.command_line=ui.Input(self.theme_manager)
-        self.bottom_bar=ui.StatusBar(self.theme_manager,"status_bar",{})
+        self.status_window=ui.Window("Status",1)
+        self.main_window=ui.Window("Main")
+        self.command_line=ui.Input()
+        self.bottom_bar=ui.StatusBar("status_bar",{})
         ui_buffer.activity_handlers.append(self.bottom_bar.update)
-        self.roster_window=ui.Window(self.theme_manager,"Roster",1)
+        self.roster_window=ui.Window("Roster",1)
 
         sp=ui.VerticalSplit(self.main_window,self.roster_window)
         sp=ui.HorizontalSplit(self.top_bar,self.status_window,sp,self.bottom_bar,self.command_line)
-        self.screen.set_content(sp)
-        self.screen.focus_window(self.main_window)
+        cjc_globals.screen.set_content(sp)
+        cjc_globals.screen.focus_window(self.main_window)
 
     def layout_irc(self):
         ui_buffer.activity_handlers=[]
-        self.top_bar=ui.StatusBar(self.theme_manager,"title_bar",{})
+        self.top_bar=ui.StatusBar("title_bar",{})
         ui_buffer.activity_handlers.append(self.top_bar.update)
-        self.status_window=ui.Window(self.theme_manager,"Status",1)
-        self.main_window=ui.Window(self.theme_manager,"Main")
-        self.command_line=ui.Input(self.theme_manager)
-        self.bottom_bar=ui.StatusBar(self.theme_manager,"status_bar",{})
+        self.status_window=ui.Window("Status",1)
+        self.main_window=ui.Window("Main")
+        self.command_line=ui.Input()
+        self.bottom_bar=ui.StatusBar("status_bar",{})
         ui_buffer.activity_handlers.append(self.bottom_bar.update)
-        self.roster_window=ui.Window(self.theme_manager,"Roster",1)
+        self.roster_window=ui.Window("Roster",1)
 
         sp=ui.VerticalSplit(self.status_window,self.roster_window)
         sp=ui.HorizontalSplit(self.top_bar,sp,self.main_window,self.bottom_bar,self.command_line)
-        self.screen.set_content(sp)
-        self.screen.focus_window(self.main_window)
+        cjc_globals.screen.set_content(sp)
+        cjc_globals.screen.focus_window(self.main_window)
 
     def layout_vertical(self):
         status_bar_params={
             "name": "CJC",
             "version": version.version,
-            "author": "Jacek Konieczny <jajcus@bnet.pl>",
+            "author": "Jacek Konieczny <jajcus@jajcus.net>",
             }
         ui_buffer.activity_handlers=[]
-        self.top_bar=ui.StatusBar(self.theme_manager,"title_bar",{})
+        self.top_bar=ui.StatusBar("title_bar",{})
         ui_buffer.activity_handlers.append(self.top_bar.update)
-        self.main_window=ui.Window(self.theme_manager,"Main")
-        self.command_line=ui.Input(self.theme_manager)
-        self.bottom_bar=ui.StatusBar(self.theme_manager,"status_bar",{})
+        self.main_window=ui.Window("Main")
+        self.command_line=ui.Input()
+        self.bottom_bar=ui.StatusBar("status_bar",{})
         ui_buffer.activity_handlers.append(self.bottom_bar.update)
-        self.roster_window=ui.Window(self.theme_manager,"Roster",1)
+        self.roster_window=ui.Window("Roster",1)
 
         sp=ui.VerticalSplit(self.main_window,self.roster_window)
         sp=ui.HorizontalSplit(self.top_bar,sp,self.bottom_bar,self.command_line)
-        self.screen.set_content(sp)
-        self.screen.focus_window(self.main_window)
+        cjc_globals.screen.set_content(sp)
+        cjc_globals.screen.focus_window(self.main_window)
         self.status_window=None
 
     def layout_horizontal(self):
         ui_buffer.activity_handlers=[]
-        self.top_bar=ui.StatusBar(self.theme_manager,"title_bar",{})
+        self.top_bar=ui.StatusBar("title_bar",{})
         ui_buffer.activity_handlers.append(self.top_bar.update)
-        self.main_window=ui.Window(self.theme_manager,"Main")
-        self.command_line=ui.Input(self.theme_manager)
-        self.bottom_bar=ui.StatusBar(self.theme_manager,"status_bar",{})
+        self.main_window=ui.Window("Main")
+        self.command_line=ui.Input()
+        self.bottom_bar=ui.StatusBar("status_bar",{})
         ui_buffer.activity_handlers.append(self.bottom_bar.update)
-        self.roster_window=ui.Window(self.theme_manager,"Roster",1)
+        self.roster_window=ui.Window("Roster",1)
         sp=ui.HorizontalSplit(self.top_bar,self.roster_window,self.main_window,self.bottom_bar,{})
-        self.screen.set_content(sp)
-        self.screen.focus_window(self.main_window)
+        cjc_globals.screen.set_content(sp)
+        cjc_globals.screen.focus_window(self.main_window)
         self.status_window=None
 
     def update_status_bars(self):
@@ -450,27 +455,27 @@ class Application(tls.TLSMixIn,jabber.Client):
         if self.bottom_bar:
             self.bottom_bar.update()
 
-    def run(self,screen):
+    def run(self, screen):
         signal.signal(signal.SIGINT,signal.SIG_IGN)
-        self.screen=screen
-        self.theme_manager=themes.ThemeManager(self)
+        cjc_globals.screen = screen
+        cjc_globals.theme_manager = themes.ThemeManager(self)
         try:
-            self.theme_manager.load()
+            cjc_globals.theme_manager.load()
         except (IOError,OSError):
             pass
-        self.theme_manager.set_default_attrs(global_theme_attrs)
-        self.theme_manager.set_default_formats(global_theme_formats)
+        cjc_globals.theme_manager.set_default_attrs(global_theme_attrs)
+        cjc_globals.theme_manager.set_default_formats(global_theme_formats)
 
-        self.status_buf=ui.TextBuffer(self.theme_manager,"Status")
-        self.status_buf.preference=self.settings["status_buffer_preference"]
+        self.status_buf = ui.TextBuffer("Status")
+        self.status_buf.preference = self.settings["status_buffer_preference"]
 
         self.set_layout(self.settings["layout"],"plain")
 
-        self.screen.update()
+        cjc_globals.screen.update()
 
         if not os.path.exists(self.home_dir):
             try:
-                os.makedirs(self.home_dir)
+                os.makedirs(self.home_dir, 0700)
             except (OSError,IOError),e:
                 self.__logger.info(e)
 
@@ -492,7 +497,7 @@ class Application(tls.TLSMixIn,jabber.Client):
             self.__logger.info("/set password your_password")
             self.__logger.info("/save")
             self.__logger.info("/connect")
-            self.__logger.info("/chat jajcus@jabber.bnet.pl CJC Rulez!")
+            self.__logger.info("/chat jajcus@jajcus.net CJC Rulez!")
             self.__logger.info("")
             self.__logger.info("If you don't have your JID yet then register on some Jabber server first using:")
             self.__logger.info("/register username@server")
@@ -534,11 +539,11 @@ class Application(tls.TLSMixIn,jabber.Client):
         logger.removeHandler(self.log_hdlr)
 
     def resize_handler(self):
-        self.screen.lock.acquire()
+        cjc_globals.screen.lock.acquire()
         layout=self.settings["layout"]
-        self.screen.set_input_handler(None)
+        cjc_globals.screen.set_input_handler(None)
         self.set_layout(layout,layout)
-        self.screen.lock.release()
+        cjc_globals.screen.lock.release()
 
     def stream_created(self,stream):
         self.send_event("stream created",stream)
@@ -691,7 +696,7 @@ class Application(tls.TLSMixIn,jabber.Client):
     def registration_form_received(self, stanza):
         register = jabber.Register(stanza.get_query())
         service_jid = stanza.get_from()
-        form_buffer = FormBuffer(self.theme_manager, {"service_name": service_jid}, "registration_form")
+        form_buffer = FormBuffer({"service_name": service_jid}, "registration_form")
         form = register.get_form()
         def callback(buf, form):
             buf.close()
@@ -706,7 +711,7 @@ class Application(tls.TLSMixIn,jabber.Client):
             if "password" in form and not form["password"].value:
                 form["password"].value = self.password
         form_buffer.set_form(form, callback)
-        self.screen.display_buffer(form_buffer)
+        cjc_globals.screen.display_buffer(form_buffer)
 
     def registration_error(self, stanza):
         err = stanza.get_error()
@@ -721,7 +726,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         self.__logger.info(u"Registration at %s successful." % (stanza.get_from(),))
            
     def process_registration_form(self, stanza, form):
-        form_buffer = FormBuffer(self.theme_manager, {"service_name": stanza.get_from()}, "registration_form")
+        form_buffer = FormBuffer({"service_name": stanza.get_from()}, "registration_form")
         def callback(buf, form):
             buf.close()
             self.submit_registration_form(form)
@@ -733,7 +738,7 @@ class Application(tls.TLSMixIn,jabber.Client):
             if "password" in form and not form["password"].value:
                 form["password"].value = self.password
         form_buffer.set_form(form, callback)
-        self.screen.display_buffer(form_buffer)
+        cjc_globals.screen.display_buffer(form_buffer)
 
     def cmd_disconnect(self,args):
         if not self.stream:
@@ -779,7 +784,7 @@ class Application(tls.TLSMixIn,jabber.Client):
                         sdef=nsdef
                     descr,typ,handler=sdef
                     val=obj.settings.get(var)
-                    if var=="password":
+                    if var=="password" and val is not None:
                         val=len(val) * '*'
                     if plugin is not None:
                         var="%s.%s" % (plugin,var)
@@ -800,7 +805,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         #self.__logger.debug("val: "+`val`)
         args.finish()
 
-        if fvar.find(".")>0:
+        if "." in fvar:
             plugin,var=fvar.split(".",1)
             try:
                 obj=self.plugins[plugin]
@@ -905,7 +910,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         if not fvar:
             return self.cmd_set(args)
 
-        if fvar.find(".")>0:
+        if "." in fvar:
             plugin,var=fvar.split(".",1)
             try:
                 obj=self.plugins[plugin]
@@ -944,13 +949,13 @@ class Application(tls.TLSMixIn,jabber.Client):
         if newval not in ("plain","icr","irc","vertical","horizontal"):
             self.settings["layout"]=oldval
             return
-        self.screen.lock.acquire()
+        cjc_globals.screen.lock.acquire()
         if self.main_window:
             main_buf=self.main_window.buffer
         else:
             main_buf=None
         getattr(self,"layout_"+newval)()
-        self.screen.lock.release()
+        cjc_globals.screen.lock.release()
         if self.status_window:
             self.status_window.set_buffer(self.status_buf)
         elif main_buf==None:
@@ -958,7 +963,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         if self.main_window and main_buf:
             self.main_window.set_buffer(main_buf)
         self.send_event("layout changed",newval)
-        self.screen.redraw()
+        cjc_globals.screen.redraw()
 
     def cmd_save(self,args):
         filename=args.shift()
@@ -1006,6 +1011,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         except IOError,e:
             self.__logger.error(u"Couldn't open config file: "+str(e))
             return 0
+        os.chmod(tmpfilename, 0600);
 
         for plugin in [None]+self.plugins.keys():
             if plugin is None:
@@ -1119,7 +1125,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         return 1
 
     def cmd_redraw(self,args):
-        self.screen.redraw()
+        cjc_globals.screen.redraw()
 
     def cmd_info(self,args):
         user=args.shift()
@@ -1181,7 +1187,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         self.__logger.info(u"  "+cmd.descr)
 
     def cmd_theme(self,args):
-        self.theme_manager.command(args)
+        cjc_globals.theme_manager.command(args)
 
     def cmd_bind(self,args):
         function=args.shift()
@@ -1223,7 +1229,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         for b in ui_buffer.buffer_list:
             if not b:
                 continue
-            formatted_list+=self.theme_manager.format_string("buffer_on_list",b.info)
+            formatted_list+=cjc_globals.theme_manager.format_string("buffer_on_list",b.info)
             i+=1
         params={
                 "buffers_on_list": formatted_list,
@@ -1262,7 +1268,7 @@ class Application(tls.TLSMixIn,jabber.Client):
             p={ "name": table.name, "priority": table.prio,
                 "bindings": self.format_keybindings,
                 "unbound": self.format_unbound_keyfunctions}
-            r+=self.theme_manager.format_string("keytable",p)
+            r+=cjc_globals.theme_manager.format_string("keytable",p)
         return r
 
     def format_keybindings(self,attr,params):
@@ -1271,7 +1277,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         for keyname,funame,desc in table.get_bindings():
             p={ "table": table.name, "key": keyname,
                 "function": funame, "description": desc}
-            r+=self.theme_manager.format_string("keybinding",p)
+            r+=cjc_globals.theme_manager.format_string("keybinding",p)
         return r
 
     def format_unbound_keyfunctions(self,attr,params):
@@ -1282,7 +1288,7 @@ class Application(tls.TLSMixIn,jabber.Client):
                 "description": f.descr}
             if f.accepts_arg:
                 p["function"]=f.name+"(<arg>)"
-            r+=self.theme_manager.format_string("keyfunction",p)
+            r+=cjc_globals.theme_manager.format_string("keyfunction",p)
         return r
 
     def exit_request(self,reason):
@@ -1379,7 +1385,9 @@ class Application(tls.TLSMixIn,jabber.Client):
             except (pyxmpp.FatalStreamError,pyxmpp.StreamEncryptionRequired),e:
                 self.state_changed.acquire()
                 try:
-                    self.__logger.error(str(e))
+                    self.__logger.error(unicode(e))
+                    if isinstance(e, pyxmpp.exceptions.TLSError):
+                        self.__logger.error(u"You may try disabling encryption: /set tls_enable false")
                     try:
                         self.stream.close()
                     except:
@@ -1414,14 +1422,14 @@ class Application(tls.TLSMixIn,jabber.Client):
         self.__logger.debug("Main loop exiting")
 
     def get_users(self,name):
-        if name.find("@")>=0:
+        if "@" in name:
             if self.roster:
                 try:
-                    ritems=self.roster.get_items_by_name(name,self.settings["case_sensitive"])
+                    ritems = self.roster.get_items_by_name(name, self.settings["case_sensitive"])
                 except KeyError:
-                    ritems=None
+                    ritems = None
                 if ritems:
-                    if len(ritems)==1:
+                    if len(ritems) == 1:
                         return [ritems[0].jid]
             try:
                 return [pyxmpp.JID(name)]
@@ -1433,44 +1441,49 @@ class Application(tls.TLSMixIn,jabber.Client):
             self.__logger.error(u"%s not found in roster" % (name,))
             return None
 
-        try:
-            ritems=self.roster.get_items_by_name(name,self.settings["case_sensitive"])
-        except KeyError:
+        ritems = self.roster.get_items_by_name(name, self.settings["case_sensitive"])
+        
+        if not ritems:
             try:
-                jid=pyxmpp.JID(name)
-                return [self.get_roster.get_item_by_jid(jid).jid()]
-            except (ValueError,pyxmpp.JIDError,KeyError):
+                jid = pyxmpp.JID(name)
+                return [self.roster[jid].jid]
+            except (ValueError, pyxmpp.JIDError, KeyError):
                 pass
             self.__logger.error(u"%s not found in roster" % (name,))
             return None
 
-        if ritems:
-            return [item.jid for item in ritems]
-        self.__logger.error(u"%s not found in roster" % (name,))
-        return None
+        return [item.jid for item in ritems]
 
     def get_user(self,name):
-        jids=self.get_users(name)
+        jids = self.get_users(name)
         if not jids:
             return None
-        if len(jids)>1:
+        if len(jids) > 1:
             self.__logger.error("ambiguous user name")
             return None
         return jids[0]
 
-    def get_best_user(self,name):
-        jids=self.get_users(name)
+    def get_best_user(self, name):
+        self.__logger.debug("get_best_user(%r)", name)
+        jids = self.get_users(name)
+        self.__logger.debug("jids = %r", name)
         if not jids:
             return None
-        if len(jids)==1:
+        if len(jids) == 1:
             return jids[0]
         my_jid = self.settings.get("jid").domain
         if my_jid:
             my_domain = pyxmpp.JID(my_jid).domain
         else:
             my_domain = None
-        decorated = [ (self.get_bare_user_info(jid, "presence_weight"), jid.domain == my_domain, jid) for jid in jids ]
+        def get_weight(jid):
+            weight = self.get_user_info(jid, "weight")
+            if weight:
+                return weight
+            return -sys.maxint
+        decorated = [ (get_weight(jid), jid.domain == my_domain, jid) for jid in jids ]
         decorated.sort()
+        self.__logger.debug("decorated = %r", decorated)
         return decorated[-1][2]
 
     def get_bare_user_info(self,jid,var=None):
@@ -1493,22 +1506,25 @@ class Application(tls.TLSMixIn,jabber.Client):
             return uinf
         return uinf.get(var)
 
-    def set_user_info(self,jid,var,val):
-        bare=jid.bare()
+    def set_user_info(self, jid, var, val):
+        self.__logger.debug("set_user_info(%r,%r,%r)" % (jid, var, val))
+        if not jid.resource:
+            return self.set_bare_user_info(jid, var, val)
+        bare = jid.bare()
         if self.user_info.has_key(bare):
-            uinf=self.user_info[bare]
+            uinf = self.user_info[bare]
             if not uinf.has_key("resources"):
-                uinf["resources"]={}
+                uinf["resources"] = {}
         else:
-            uinf={"resources":{},"jid":bare}
-            self.user_info[bare]=uinf
+            uinf = {"resources": {}, "jid": bare}
+            self.user_info[bare] = uinf
 
         if uinf["resources"].has_key(jid.resource):
-            fuinf=uinf["resources"][jid.resource]
+            fuinf = uinf["resources"][jid.resource]
         else:
-            fuinf={"jid":jid}
-            uinf["resources"][jid.resource]=fuinf
-        fuinf[var]=val
+            fuinf = {"jid":jid}
+            uinf["resources"][jid.resource] = fuinf
+        fuinf[var] = val
 
     def set_bare_user_info(self,jid,var,val):
         bare=jid.bare()
@@ -1536,13 +1552,13 @@ class Application(tls.TLSMixIn,jabber.Client):
         elif state=="connected":
             self.__logger.info(u"Connected to %s:%i." % (arg[0],arg[1]))
         elif state=="authenticating":
-            self.__logger.info(u"Authenticating as %s..." % (arg,))
+            self.__logger.info(u"Authenticating as %s..." % (unicode(arg),))
         elif state=="binding":
-            self.__logger.info(u"Binding to resource %s..." % (arg,))
+            self.__logger.info(u"Binding to resource %s..." % (unicode(arg),))
         elif state=="authorized":
-            self.__logger.info(u"Authorized as %s." % (arg,))
+            self.__logger.info(u"Authorized as %s." % (unicode(arg),))
         elif state=="tls connecting":
-            self.__logger.info(u"Doing TLS handshake with %s." % (arg,))
+            self.__logger.info(u"Doing TLS handshake with %s." % (unicode(arg),))
 
     def show_error(self,s):
         self.status_buf.append_themed("error",s)
@@ -1660,7 +1676,7 @@ ui.CommandTable("global",10,(
 
 def usage():
     print
-    print "Console Jabber Client (c) 2003-2005 Jacek Konieczny <jajcus@bnet.pl>"
+    print "Console Jabber Client (c) 2003-2009 Jacek Konieczny <jajcus@jajcus.net>"
     print
     print "Usage:"
     print "  %s [OPTIONS]" % (sys.argv[0],)
