@@ -1,5 +1,5 @@
 # Console Jabber Client
-# Copyright (C) 2004-2005  Jacek Konieczny
+# Copyright (C) 2004-2009  Jacek Konieczny
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -16,12 +16,10 @@
 
 class Widget:
     def __init__(self):
-        self.screen=None
         self.parent=None
 
     def set_parent(self,parent):
         self.parent=parent
-        self.screen=parent.screen
         self.x,self.y,self.w,self.h=self.parent.place(self)
 
     def get_height(self):
