@@ -1,5 +1,5 @@
 # Console Jabber Client
-# Copyright (C) 2004-2005  Jacek Konieczny
+# Copyright (C) 2004-2009  Jacek Konieczny
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -17,6 +17,7 @@
 import string
 import curses
 import os
+import logging
 
 import pyxmpp
 from pyxmpp.jabber import muc,delay
@@ -24,6 +25,7 @@ from cjc import ui
 from cjc.ui.form_buffer import FormBuffer
 from cjc.plugin import PluginBase
 from cjc import common
+from cjc import cjc_globals
 
 theme_attrs=(
     ("muc.me", curses.COLOR_YELLOW,curses.COLOR_BLACK,curses.A_BOLD, curses.A_UNDERLINE),
@@ -68,12 +70,13 @@ class Room(muc.MucRoomHandler):
             "room":self.room,
             "me":self.me,
         }
-        self.buffer=ui.TextBuffer(plugin.cjc.theme_manager,self.fparams,
+        self.buffer=ui.TextBuffer(self.fparams,
                 "muc.descr","muc buffer",self)
         self.buffer.preference=plugin.settings["buffer_preference"]
         self.buffer.user_input=self.user_input
         self.buffer.get_completion_words=self.get_completion_words
         self.buffer.append_themed("muc.joining",self.fparams)
+        self.buffer._muc_room = self
         self.buffer.update()
 
     def user_format_params(self,user):
@@ -151,16 +154,17 @@ class Room(muc.MucRoomHandler):
         self.buffer.update()
         return
 
-    def user_joined(self,user,stanza):
-        fparams=self.user_format_params(user)
-        d=delay.get_delay(stanza)
+    def user_joined(self, user, stanza):
+        self.plugin.cjc.set_user_info(user.room_jid, "nick", user.nick)
+        fparams = self.user_format_params(user)
+        d = delay.get_delay(stanza)
         if d:
-            fparams["timestamp"]=d.get_datetime_local()
-        self.plugin.cjc.send_event("groupchat user joined",user.nick)
+            fparams["timestamp"] = d.get_datetime_local()
+        self.plugin.cjc.send_event("groupchat user joined", user.nick)
         if user.same_as(self.room_state.me):
-            self.buffer.append_themed("muc.me_joined",fparams)
+            self.buffer.append_themed("muc.me_joined", fparams)
         else:
-            self.buffer.append_themed("muc.joined",fparams)
+            self.buffer.append_themed("muc.joined", fparams)
         self.buffer.update()
         return
 
@@ -249,9 +253,9 @@ class Room(muc.MucRoomHandler):
             self.room_state.request_configuration_form()
 
     def configuration_form_received(self, form):
-        form_buffer = FormBuffer(self.plugin.cjc.theme_manager, self.fparams, "muc.conf_descr")
+        form_buffer = FormBuffer(self.fparams, "muc.conf_descr")
         form_buffer.set_form(form, self.configuration_callback)
-        self.plugin.cjc.screen.display_buffer(form_buffer)
+        cjc_globals.screen.display_buffer(form_buffer)
 
     def configuration_callback(self, form_buffer, form):
         form_buffer.close()
@@ -303,9 +307,10 @@ class Room(muc.MucRoomHandler):
         return 1
 
     def cmd_who(self, args):
-        nicks = ','.join(self.room_state.users.keys())
-        self.buffer.append(nicks)
+        nicks = u','.join(self.room_state.users.keys())
+        self.buffer.append(nicks + u"\n")
         self.buffer.update()
+        
     def cmd_nick(self,args):
         new_nick=args.all()
         if not args:
@@ -317,6 +322,23 @@ class Room(muc.MucRoomHandler):
         self.room_state.change_nick(new_nick)
         return 1
 
+    def cmd_query(self, args):
+        nick = args.shift()
+        if not nick:
+            raise CommandError,"No nickname given"
+        if nick not in self.room_state.users:
+            self.buffer.append_themed("error", "No '%s' in this room", nick)
+            self.buffer.update()
+            return 1
+        user = self.room_state.users[nick]
+        rest = args.all()
+        args = u'"%s"' % (user.room_jid.as_unicode().replace('"', '\\"'), )
+        if rest:
+            args = ui.CommandArgs(args + u" " + rest)
+        else:
+            args = ui.CommandArgs(args)
+        ui.run_command("chat", args)
+
     def cmd_leave(self,args):
         a=args.get()
         if a:
@@ -339,6 +361,31 @@ class Room(muc.MucRoomHandler):
     def get_completion_words(self):
         return [nick+u":" for nick in self.room_state.users.keys()]
 
+class MucNickCompletion(ui.Completion):
+    def __init__(self, app):
+        ui.Completion.__init__(self)
+        self.app = app
+        self.__logger=logging.getLogger("plugins.muc.MucNickCompletion")
+
+    def complete(self, word):
+        self.__logger.debug("MucNickCompletion.complete(self,%r)" % (word,))
+        active_window = cjc_globals.screen.active_window
+        if not active_window:
+            return "", []
+        active_buffer = active_window.buffer
+        if not active_window.buffer:
+            return "", []
+        try:
+            muc_room = active_buffer._muc_room
+        except AttributeError:
+            return "", []
+
+        matches=[]
+        for w in muc_room.room_state.users.keys():
+            if w.startswith(word):
+                matches.append( [w, 1] )
+        return self.make_result("", word,matches)
+
 ui.CommandTable("muc buffer",51,(
     ui.Command("me",Room.cmd_me,
         "/me text",
@@ -356,6 +403,10 @@ ui.CommandTable("muc buffer",51,(
         "/who ",
         "Lists users in this room",
         ("text",)),
+    ui.Command("query", Room.cmd_query,
+        "/query nick",
+        "Starts a private chat",
+        ("muc_nick",)),
     ui.CommandAlias("topic","subject"),
     ui.Command("leave",Room.cmd_leave,
         "/leave [jid]",
@@ -372,8 +423,8 @@ ui.CommandTable("muc buffer",51,(
 class Plugin(PluginBase):
     def __init__(self,app,name):
         PluginBase.__init__(self,app,name)
-        app.theme_manager.set_default_attrs(theme_attrs)
-        app.theme_manager.set_default_formats(theme_formats)
+        cjc_globals.theme_manager.set_default_attrs(theme_attrs)
+        cjc_globals.theme_manager.set_default_formats(theme_formats)
         self.available_settings={
             "log_filename": ("Where messages should be logged to",(unicode,None)),
             "log_format_in": ("Format of incoming message log entries",(unicode,None)),
@@ -391,6 +442,7 @@ class Plugin(PluginBase):
         app.add_event_handler("day changed",self.ev_day_changed)
         ui.activate_cmdtable("muc",self)
         self.room_manager=None
+        MucNickCompletion(app).register("muc_nick")
 
     def cmd_join(self,args):
         if not self.cjc.stream:
@@ -424,7 +476,7 @@ class Plugin(PluginBase):
         else:
             room_handler=Room(self,room_jid,self.cjc.stream.me)
             self.room_manager.join(room_jid,nick,room_handler)
-        self.cjc.screen.display_buffer(room_handler.buffer)
+        cjc_globals.screen.display_buffer(room_handler.buffer)
 
     def cmd_leave(self,args):
         room_jid=args.shift()
@@ -471,8 +523,8 @@ class Plugin(PluginBase):
             d["peer"]=sender
         else:
             d["peer"]=recipient
-        filename=self.cjc.theme_manager.substitute(filename,d)
-        s=self.cjc.theme_manager.substitute(format,d)
+        filename=cjc_globals.theme_manager.substitute(filename,d)
+        s=cjc_globals.theme_manager.substitute(format,d)
         try:
             dirname=os.path.split(filename)[0]
             if dirname and not os.path.exists(dirname):
