@@ -1,5 +1,5 @@
 # Console Jabber Client
-# Copyright (C) 2004-2005  Jacek Konieczny
+# Copyright (C) 2004-2009  Jacek Konieczny
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -21,6 +21,7 @@ import pyxmpp
 
 from cjc.plugin import PluginBase
 from cjc import ui
+from cjc import cjc_globals
 
 theme_formats=(
     ("presence.available","%[info][%(T:timestamp)s] %(J:user)s (%(J:user:rostername)s) is %(J:user:show)s: %(J:user:status)s\n"),
@@ -39,7 +40,7 @@ show_weight = {"xa": 0, "away": 1, "dnd": 2, None: 3, "chat": 4}
 class Plugin(PluginBase):
     def __init__(self,app,name):
         PluginBase.__init__(self,app,name)
-        app.theme_manager.set_default_formats(theme_formats)
+        cjc_globals.theme_manager.set_default_formats(theme_formats)
         self.available_settings={
             "priority": ("Priority of current resource",int),
             "chat_priority": ("Priority of current resource in ready-for-chat"
@@ -99,41 +100,41 @@ class Plugin(PluginBase):
     def info_resources(self,k,v):
         if not v:
             return None
-        resources=[]
+        resources = []
         for r in v.keys():
-            p=v[r].get("presence")
-            if p is None or (p.get_type() and p.get_type()!="available"):
+            p = v[r].get("presence")
+            if p is None or (p.get_type() and p.get_type() != "available"):
                 continue
-
             if not r:
-                resources.append(u"<empty>")
-            else:
-                resources.append(r)
+                r = u"<empty>"
+            r += " (prio=%i)" % (p.get_priority(), )
+            resources.append(r)
         if resources:
-            return "Available resources",string.join(resources,",")
+            return "Available resources", string.join(resources, ", ")
 
     def info_presence(self,k,v):
         if not v:
             return None
 
-        name="Presence"
-        if not v.get_type() or v.get_type()=="available":
-            value="Available"
+        name = "Presence"
+        if not v.get_type() or v.get_type() == "available":
+            value = "Available"
+            value += " (prio=%i)" % (v.get_priority(), )
             if v.get_show():
-                value+=" [%s]" % (v.get_show(),)
+                value += " [%s]" % (v.get_show(), )
             if v.get_status():
-                value+=" %s" % (v.get_status(),)
-        elif v.get_type()=="unavailable":
-            value="Not Available"
+                value += " %s" % (v.get_status(), )
+        elif v.get_type() == "unavailable":
+            value = "Not Available"
             if v.get_status():
-                value+=" %s" % (v.get_status(),)
-        elif v.get_type()=="error":
-            value="Error"
-            e=v.get_error()
+                value += " %s" % (v.get_status(), )
+        elif v.get_type() == "error":
+            value = "Error"
+            e = v.get_error()
             if e:
-                c=e.get_condition()
+                c = e.get_condition()
                 if c:
-                    value+=": %s" % (c.serialize(),)
+                    value += ": %s" % (c.serialize(), )
         else:
             return None
         return name,value
@@ -175,7 +176,8 @@ class Plugin(PluginBase):
             return
 
         if not self.away_saved_presence:
-            self.away_saved_presence=p
+            self.away_saved_presence = p.copy()
+            self.away_saved_presence.set_to(None)
 
         insert_time=False
 
@@ -205,7 +207,7 @@ class Plugin(PluginBase):
             return
 
         self.cjc.add_event_handler("keypressed",self.ev_keypressed)
-        if insert_time and status.find("%i")>=0:
+        if insert_time and "%i" in status:
             p=pyxmpp.Presence(priority=prio, show=show, status=status % (idle,))
         else:
             p=pyxmpp.Presence(priority=prio, show=show, status=status)
@@ -381,12 +383,12 @@ class Plugin(PluginBase):
 
     def presence_available(self,stanza):
         fr=stanza.get_from()
-        p=self.cjc.get_user_info(fr,"presence")
-        self.cjc.set_user_info(fr,"presence",stanza.copy())
+        p=self.cjc.get_user_info(fr, "presence")
+        self.cjc.set_user_info(fr, "presence", stanza.copy())
         self.compute_current_resource(fr.bare())
-        self.cjc.send_event("presence changed",fr)
+        self.cjc.send_event("presence changed", fr)
         if (not p or p!=stanza) and self.settings.get("show_changes"):
-            self.cjc.status_buf.append_themed("presence.available",{"user":fr})
+            self.cjc.status_buf.append_themed("presence.available", {"user":fr})
             self.cjc.status_buf.update()
         else:
             self.debug(fr.as_unicode()+u" is unavailable")
@@ -395,7 +397,7 @@ class Plugin(PluginBase):
     def presence_unavailable(self,stanza):
         fr=stanza.get_from()
         if self.cjc.get_user_info(fr) and self.settings.get("show_changes"):
-            self.cjc.status_buf.append_themed("presence.unavailable",{"user":fr})
+            self.cjc.status_buf.append_themed("presence.unavailable", {"user":fr})
             self.cjc.status_buf.update()
         else:
             self.debug(fr.as_unicode()+u" is unavailable")
@@ -404,31 +406,34 @@ class Plugin(PluginBase):
         self.cjc.send_event("presence changed",fr)
         return 1
 
-    def compute_current_resource(self,jid):
-        resources=self.cjc.get_bare_user_info(jid,"resources")
+    def compute_current_resource(self, jid):
+        resources = self.cjc.get_bare_user_info(jid, "resources")
         if not resources:
-            p=self.cjc.get_bare_user_info(jid,"presence")
-            if p and p.get_type()!="error" and p.get_jid().resource:
-                self.cjc.set_bare_user_info(jid,"presence",None)
+            p = self.cjc.get_bare_user_info(jid, "presence")
+            if p and p.get_type() != "error" and p.get_from().resource:
+                self.cjc.set_bare_user_info(jid, "presence", None)
             self.cjc.set_bare_user_info(jid, "weight", None)
             return
-        presence=None
-        max_prio=-129
-        for r,d in resources.items():
-            fjid=pyxmpp.JID(jid.node,jid.domain,r,check=0)
+        presence = None
+        max_prio = -129
+        for r, d in resources.items():
+            fjid = pyxmpp.JID(jid.node, jid.domain, r, check = 0)
             if not d.has_key("presence"):
                 continue
-            p=d["presence"]
+            p = d["presence"]
             if not p:
                 continue
-            typ=p.get_type()
-            if typ and p.get_type()!="available":
+            typ = p.get_type()
+            if typ and typ != "available":
                 continue
-            prio=p.get_priority()
-            if prio>max_prio:
-                max_prio=prio
-                presence=p
-        weight = show_weight.get(p.get_show(), show_weight[None])*1000 + max_prio
+            prio = p.get_priority()
+            if prio > max_prio:
+                max_prio = prio
+                presence = p
+        if presence:
+            weight = show_weight.get(presence.get_show(), show_weight[None]) * 1000 + max_prio
+        else:
+            weight = 0
         if max_prio < 0:
             weight -= 10000
         self.cjc.set_bare_user_info(jid, "presence", presence)
@@ -440,7 +445,7 @@ class Plugin(PluginBase):
             self.debug("Ignoring own presence subscription request")
             return
         reason=stanza.get_status()
-        buf=ui.TextBuffer(self.cjc.theme_manager,{"user":fr},"presence.subscribe_buffer")
+        buf=ui.TextBuffer({"user":fr},"presence.subscribe_buffer")
         buf.preference=self.settings["buffer_preference"]
         buf.append_themed("presence.subscribe",{"user":fr,"reason":reason})
         stanza_copy = stanza.copy()
@@ -448,7 +453,7 @@ class Plugin(PluginBase):
             return self.subscribe_decision(response, stanza_copy, buf)
         buf.ask_question("Accept?", "boolean", None, callback, None, None, 1)
         if self.settings.get("auto_popup"):
-            self.cjc.screen.display_buffer(buf)
+            cjc_globals.screen.display_buffer(buf)
 
     def subscribe_decision(self, accept, stanza, buf):
         fr=stanza.get_from()
